package jp.ikebun.ivs_player2

import android.app.Activity
import android.view.View
import android.widget.FrameLayout
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import io.flutter.plugin.common.PluginRegistry.ViewDestroyListener
import io.flutter.plugin.common.PluginRegistry

import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory
import io.flutter.plugin.common.StandardMessageCodec

import com.amazonaws.ivs.player.Player
import com.amazonaws.ivs.player.Player.State
import com.amazonaws.ivs.player.PlayerView
import android.net.Uri
import android.content.Context
import io.flutter.embedding.engine.plugins.FlutterPlugin
import android.util.DisplayMetrics

import com.amazonaws.ivs.player.Cue
import com.amazonaws.ivs.player.TextCue
import com.amazonaws.ivs.player.PlayerException
import com.amazonaws.ivs.player.Quality
import android.widget.TextView
import android.view.Gravity
import android.graphics.Color

import android.util.Log


class NativeViewFactory(private val context: Context,private val plugin: IvsPlayer2Plugin) : PlatformViewFactory(StandardMessageCodec.INSTANCE) {
    override fun create(context: Context, viewId: Int, args: Any?): PlatformView {
        val params = args as? Map<String, Any>
        val url = params?.get("url") as? String?:""
        val nativeView = NativeView(context,url)
        plugin.nativeView = nativeView
        return nativeView
    }
}

class NativeView(context: Context,private val url: String) : PlatformView {
    private val frameLayout = FrameLayout(context)
    private val view: PlayerView = PlayerView(context).apply {
        setControlsEnabled(false)
    }
    private val subtitleView: SubtitleView = SubtitleView(context).apply {
        setBackgroundColor(Color.parseColor("#80000000"))
        setTextColor(Color.WHITE)
        gravity = Gravity.CENTER_HORIZONTAL 
       
    }

    init {
        setCaptionVisibility(false)
        setMuted(false)
    view?.player?.let{player ->
    player.load(Uri.parse(url))
        player.addListener(object: Player.Listener(){
            override fun onCue(cue: Cue) {
                if (cue is TextCue) {
                    val subtitleText = cue.text
                     Log.d("Subtitle", "Received subtitle: $subtitleText")
                     subtitleView.text = subtitleText
                }
            }
            // 他のイベントリスナーの実装...
            override fun onDurationChanged(p0: Long) {
            // ここにonDurationChangedが呼ばれたときの処理を書く
        }
        override fun onStateChanged(p0: Player.State) {
            // ここにonStateChangedが呼ばれたときの処理を書く
        }
        override fun onError(p0: PlayerException) {
            // ここにonErrorが呼ばれたときの処理を書く
        }
        override fun onRebuffering() {
            // ここにonRebufferingが呼ばれたときの処理を書く
        }
        override fun onSeekCompleted(p0: Long) {
            // ここにonSeekCompletedが呼ばれたときの処理を書く
        }
        override fun onVideoSizeChanged(p0: Int, p1: Int) {
            // ここにonVideoSizeChangedが呼ばれたときの処理を書く
        }
        override fun onQualityChanged(p0: Quality) {
            // ここにonQualityChangedが呼ばれたときの処理を書く
        }

        })
        frameLayout.addView(view, FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.MATCH_PARENT))
        val subtitleLayoutParams = FrameLayout.LayoutParams(FrameLayout.LayoutParams.MATCH_PARENT, FrameLayout.LayoutParams.WRAP_CONTENT).apply {
            gravity = Gravity.CENTER_HORIZONTAL 
        }
        frameLayout.addView(subtitleView, subtitleLayoutParams)
        
        player.play()
        }
    }

    override fun getView(): View {
        return frameLayout
    }

    override fun dispose() {}

    fun setCaptionVisibility(isCaptionEnabled: Boolean){
        subtitleView?.let{
            it.visibility = if (isCaptionEnabled) View.VISIBLE else View.GONE
        }
    }

    fun setMuted(isMuted: Boolean){
        view?.player?.let{
            it.setMuted(isMuted)
        }
    }
}

class IvsPlayer2Plugin: FlutterPlugin, MethodCallHandler, ActivityAware{
    private var activity: Activity? = null
    private lateinit var channel : MethodChannel
    private var player: Player? = null
    private lateinit var context: Context
    private var playerView: PlayerView? = null
    private var isMuted: Boolean = false
    private var isCaptionEnabled: Boolean = false
    private var subtitleView: SubtitleView? = null
    var nativeView: NativeView? = null


    

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "ivs_player2")
        channel.setMethodCallHandler(this)
        context = flutterPluginBinding.applicationContext
        playerView = PlayerView(context)
        flutterPluginBinding
            .platformViewRegistry
            .registerViewFactory("native-view", NativeViewFactory(flutterPluginBinding.applicationContext,this))

    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activity = binding.activity

    }

    override fun onDetachedFromActivityForConfigChanges() {
        activity = null
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        onAttachedToActivity(binding)
    }

    override fun onDetachedFromActivity() {
        activity = null
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
         if (call.method == "muted") {
           
            
                    isMuted = !isMuted
              
                    nativeView?.setMuted(isMuted)
                
            
        } else if(call.method == "caption"){
    
                    isCaptionEnabled = !isCaptionEnabled
                    nativeView?.setCaptionVisibility(isCaptionEnabled)
                
            
        } else {
            result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
        player?.release()

    }
}



    class SubtitleView(context: Context) : TextView(context) {
    init {
        this.textAlignment = View.TEXT_ALIGNMENT_CENTER
        this.setTextColor(Color.WHITE)
        this.setBackgroundColor(Color.parseColor("#80000000"))
        //this.gravity = Gravity.CENTER_HORIZONTAL | Gravity.TOP
        this.layoutParams = FrameLayout.LayoutParams(
            FrameLayout.LayoutParams.MATCH_PARENT,
            FrameLayout.LayoutParams.WRAP_CONTENT,
            Gravity.CENTER_HORIZONTAL 
        )
    }
}