import Flutter
import UIKit
import AmazonIVSPlayer
import AVFoundation


class MyNativeView: NSObject, FlutterPlatformView,IVSPlayer.Delegate {
    private var _view: UIView
    private var player: IVSPlayer?
    private var playerView: IVSPlayerView?
    private var subtitleLabel: UILabel? 


    init(frame: CGRect, viewIdentifier viewId: Int64, arguments args: Any?, binaryMessenger messenger: FlutterBinaryMessenger?) {
        _view = UIView(frame: frame)
        super.init()

        print("受け取った引数: \(String(describing: args))")
       
       guard let argumentMap = args as? [String: Any], let urlString = argumentMap["url"] as? String, let width = argumentMap["width"] as? Double, let height = argumentMap["height"] as? Double else {
            print("URLが提供されていません")
            return
        }
        
        playStream(url: urlString,width: width, height: height)
        setCaptionVisibility(isVisible: false)
        
        
        
        
    }

    func view() -> UIView {
        return _view
    }

    func setMuted(isMuted: Bool){
        if let player = player {
            player.muted = isMuted
        }
    }

    func setCaptionVisibility(isVisible: Bool){
        if let subtitleLabel = subtitleLabel {
        !isVisible ? subtitleLabel.removeFromSuperview() : _view.addSubview(subtitleLabel)
        }
    }


    private func playStream(url: String,width: Double, height: Double) {
        do {
            try AVAudioSession.sharedInstance().setCategory(.playback, mode: .default, options: [])
            try AVAudioSession.sharedInstance().setActive(true)
        } catch {
            print("Failed to set audio session category. Error: \(error)")
        }
    player = IVSPlayer()
    player?.delegate = self

    let playerViewFrame = CGRect(
        x: 0,
        y:0,
        width: width,
        height: height
    )
    playerView = IVSPlayerView(frame:playerViewFrame)
    playerView?.player = player
    playerView?.backgroundColor = .black.withAlphaComponent(0.5) // 背景色を黒に設定
    player?.load(URL(string: url)!)

    // playerViewをviewに追加
if let playerView = playerView {
    _view.addSubview(playerView)
}

// サブタイトルラベルの設定
let subtitleLabelHeight: CGFloat = 50
let bottomSafeAreaInset = _view.safeAreaInsets.bottom // セーフエリアの下部のマージンを取得
let subtitleLabelYPosition = UIScreen.main.bounds.height - subtitleLabelHeight - bottomSafeAreaInset

subtitleLabel = UILabel(frame: CGRect(x: 0, y: 0, width: UIScreen.main.bounds.width, height: 20))
subtitleLabel?.textAlignment = .center
subtitleLabel?.textColor = .white
subtitleLabel?.backgroundColor = .black.withAlphaComponent(0.5) // 背景色を半透明の黒に設定
subtitleLabel?.numberOfLines = 0

// サブタイトルラベルをplayerViewではなく、viewに追加
if let subtitleLabel = subtitleLabel {
    playerView?.addSubview(subtitleLabel)
    playerView?.bringSubviewToFront(subtitleLabel)
}

   
    
    
}

public func player(_ player: IVSPlayer, didChangeState state: IVSPlayer.State) {
    if state == .ready {
        print("Player ready")
        player.play()
    }
}

public func player(_ player: IVSPlayer, didOutputCue cue: IVSCue) {
    if let textCue = cue as? IVSTextCue {
        let subtitleText = textCue.text
        print("Subtitle text: \(subtitleText)")
        DispatchQueue.main.async { [weak self] in
                self?.subtitleLabel?.text = subtitleText // UILabelに字幕テキストを設定
                // 幅を固定値で指定
                let fixedWidth = UIScreen.main.bounds.width
// sizeThatFitsを使用して、指定した幅での適切な高さを計算
let maxSize = CGSize(width: fixedWidth, height: CGFloat.greatestFiniteMagnitude)
let requiredSize = self?.subtitleLabel?.sizeThatFits(maxSize)

// 計算された高さを使用して、ラベルのフレームを更新
if let requiredHeight = requiredSize?.height {
    self?.subtitleLabel?.frame = CGRect(x: 0, y: 0, width: fixedWidth, height: requiredHeight)
}
        }
    }
}



}


class MyNativeViewFactory: NSObject, FlutterPlatformViewFactory {
    private var messenger: FlutterBinaryMessenger
    private var plugin: IvsPlayer2Plugin
    

    init(messenger: FlutterBinaryMessenger,plugin: IvsPlayer2Plugin) {
        self.messenger = messenger
        self.plugin  = plugin
        super.init()
    }
    public func createArgsCodec() -> FlutterMessageCodec & NSObjectProtocol {
    return FlutterStandardMessageCodec.sharedInstance()
}

    func create(withFrame frame: CGRect, viewIdentifier viewId: Int64, arguments args: Any?) -> FlutterPlatformView {
        print(args ?? "なし")
        let view = MyNativeView(frame: frame, viewIdentifier: viewId, arguments: args, binaryMessenger: messenger)
        plugin.nativeView = view
        return view
    }
}





public class IvsPlayer2Plugin: NSObject, FlutterPlugin, IVSPlayer.Delegate {
    private var isMuted: Bool = false
    private var isCaptionVisible: Bool = false
    var nativeView: MyNativeView? = nil

    public static func register(with registrar: FlutterPluginRegistrar) {     
        let channel = FlutterMethodChannel(name: "ivs_player2", binaryMessenger: registrar.messenger())
        let instance = IvsPlayer2Plugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
        let factory = MyNativeViewFactory(
            messenger: registrar.messenger(),
            plugin: instance
        )
        registrar.register(factory, withId: "native-view")

        
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "getPlatformVersion":
            result("iOS " + UIDevice.current.systemVersion)
        case "caption":
        isCaptionVisible = !isCaptionVisible
            if let nativeView = nativeView {
                nativeView.setCaptionVisibility(isVisible: isCaptionVisible)
            }
        case "muted":
        isMuted = !isMuted
      
        if let nativeView = nativeView {
            nativeView.setMuted(isMuted: isMuted)
        }
        default:
            result(FlutterMethodNotImplemented)
        }
    }

   

//     public func player(_ player: IVSPlayer, didChangeState state: IVSPlayer.State) {
//     if state == .ready {
//         print("Player ready")
//         player.play()
//     }
// }

// public func player(_ player: IVSPlayer, didFailWithError error: Error) {
//     print("Player error: \(error.localizedDescription)")
// }

// public func player(_ player: IVSPlayer, didOutputCue cue: IVSCue) {
//     if let textCue = cue as? IVSTextCue {
//         let subtitleText = textCue.text
//         print("Subtitle text: \(subtitleText)")
//         DispatchQueue.main.async { [weak self] in
//                 self?.subtitleLabel?.text = subtitleText // UILabelに字幕テキストを設定
//         }
//     }
// }

}
