import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class IvsPlayer2 {
  static const MethodChannel _channel = MethodChannel('ivs_player2');

  Future<String?> getPlatformVersion() async {
    final version = await _channel.invokeMethod<String>('getPlatformVersion');
    return version;
  }

  Future<void> playStream(String url) async {
    await _channel.invokeMethod('playStream', <String, dynamic>{
      'url': url,
    });
  }

  Future<void> muted() async {
    return await _channel.invokeMethod('muted');
  }

  Future<void> caption() async {
    return await _channel.invokeMethod('caption');
  }

  Widget player(String url) {
    switch (defaultTargetPlatform) {
      case TargetPlatform.android:
        return AndroidView(
          viewType: 'native-view',
          creationParams: <String, dynamic>{
            'url': url,
          },
          creationParamsCodec: const StandardMessageCodec(),
        );
      case TargetPlatform.iOS:
        return LayoutBuilder(
            builder: (BuildContext context, BoxConstraints constraints) {
          double availableWidth = constraints.maxWidth;
          double availableHeight = constraints.maxHeight;
          return UiKitView(
            viewType: 'native-view',
            creationParams: <String, dynamic>{
              'url': url,
              'width': availableWidth,
              'height': availableHeight,
            },
            creationParamsCodec: const StandardMessageCodec(),
          );
        });
      default:
        return const Text("Not supported");
    }
  }
}
